#<<<
## no critic
use utf8;
package Stanford::Certificate;

# Created by DBIx::Class::Schema::Loader
# DO NOT MODIFY THE FIRST PART OF THIS FILE

use strict;
use warnings;

use base 'DBIx::Class::Schema';

__PACKAGE__->load_namespaces;

## use critic
#>>>

# Created by DBIx::Class::Schema::Loader v0.07042 @ 2016-04-24 12:24:07
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:uQPpyF41jyY/K3kOzkFySA

our $VERSION = '0.01' ;

# You can replace this text with custom content, and it will be preserved
# on regeneration


use Carp ;
use Data::Dumper ;
use Stanford::Orange::Util qw/ trim tempfile / ;

# Load the configuration:
use Stanford::Certificate::Config ;

use Stanford::Certificate::Result::Request ;
use Stanford::Certificate::Result::LegacyExpiration ;

# Make a fake certificate request
sub make_fake_request {
    my ($schema) = @_ ;

    my $request = $schema->resultset('Request')->new({}) ;

    $request->csr(undef) ;
    $request->duration(3) ;
    $request->requester_sunetid('bogusbogus') ;
    $request->requester_fullname('Joe Bogus') ;
    $request->requester_email('bogusbogus@stanford.edu') ;
    $request->requester_affiliation(undef) ;
    $request->requester_department('IT Services') ;
    $request->contact_email('certrequesters@baddomain.qqq') ;
    $request->requester_comments('Make it snappy, bud!') ;
    $request->alternatives('Joe Bogus') ;
    $request->approval_contacts('adamhl@stanford.edu') ;

    my $server_type = $schema->resultset('ServerType')->find(
        { name => 'Apache/ModSSL', },
        ) ;
    $request->server_type($server_type) ;

    my $certificate_type = $schema->resultset('CertificateType')->find(
        { name => 'normal', },
        ) ;
    $request->certificate_type($certificate_type) ;

    return $request ;
}



# Sample DSN's:
#
#   dbi:SQLite:dbname=/tmp/test.sqlite
#   mysql://root:secret@localhost/cert_db
#
# Rather than "connect" we "attach":
sub attach {
    my ($yml_file) = @_ ;

    # If the YAML file is not supplied, use the one from the
    # configuration.
    if (!$yml_file) {
        my $config = Stanford::Certificate::Config::instance() ;
        $yml_file = $config->db_connect_yaml_file() ;
    }

    if (!(-e $yml_file)) {
        croak "no YAML file '$yml_file' found" ;
    }

    # Read from the YML file.
    use YAML::Tiny ;

    # Create a YAML file
    my $yaml = YAML::Tiny->new ;

    # Open the config
    $yaml = YAML::Tiny->read($yml_file) ;

    # Read the values.
    my $dsn      = $yaml->[0]->{'perl_dsn'} ;
    my $username = $yaml->[0]->{'username'} ;
    my $password = $yaml->[0]->{'password'} ;

    my $schema = Stanford::Certificate->connect($dsn, $username, $password) ;

    return $schema ;
}

1 ;

__END__

=head1 NAME

Stanford::Certificate - InCommon Certificate module

=head1 VERSION

Version 0.01

=head1 ABSTRACT

The Stanford::Certificate class ...

=head1 AUTHOR

Adam Lewenberg, C<< <adamhl at stanford.edu> >>

=head1 BUGS

Please report any bugs or feature requests
to C<adamhl at stanford.edu>.

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Stanford::Certificate

=head1 COPYRIGHT & LICENSE

Copyright 2008 Stanford University, all rights reserved.

=cut

