package Stanford::Certificate::Mail ;

use strict;
use warnings;
use autodie qw(open close) ;

require Exporter;
use base qw/ Exporter / ;

use Stanford::Certificate::Config ;
use Stanford::Certificate::Log ;
use Carp ;

our @EXPORT_OK = qw(
  send_email
);


sub send_email
{
  return send_email_using_sendmail(@_) ;
}

# We override the 'to' field by $config->{'to_address_override'} if
# $config->{'to_address_override'} has a non-empty value.
sub send_email_ssl
{
  # uncoverable subroutine
  my $config = Stanford::Certificate::Config::instance() ;

  my %args = (
                'to'      => undef,
                'from'    => $config->{'from_email_address'},
                'subject' => 'NO SUBJECT',
                'body'    => 'NO MESSAGE BODY',
                @_,
             ) ;

  if (!$args{'to'})
  {
    croak "cannot send mail without a To: address" ;
  }

  use Net::SMTP;
  use Net::SMTP::TLS;
  use MIME::Lite ;

  my $smtp_server      = 'smtp.stanford.edu' ;
  my $smtp_server_port = 25 ;

  # Do we override the To: address?
  if (    $config->{'to_email_override'}
      && ($config->{'to_email_override'} ne $args{'to'})
     )
  {
    # Yes, override.
    $args{'subject'} = $args{'subject'} . " (address " .
                       $args{'to'} . " overridden)" ;
    $args{'to'}      = $config->{'to_email_override'} ;
  }

  # Get the "From:" address
  if (!$args{'from'})
  {
      croak "no From: address; cannot proceed" ;
  }

  my $msg = MIME::Lite->new(
    From    => $args{'from'},
    To      => $args{'to'},
    Subject => $args{'subject'},
    Data    => $args{'body'},
  ) ;


  my $mailer = Net::SMTP::TLS->new(
    $smtp_server,
    Hello   => $smtp_server_port,
    Port    => 25,
    Debug   => 1,
    # User    => 'username',
    # Password=> 'password',
  ) ;

  $mailer->mail($args{'from'});
  $mailer->to($args{'to'});
  $mailer->data() ;
  $mailer->datasend($msg->as_string());
  $mailer->dataend() ;
  $mailer->quit() ;

  Stanford::Certificate::Log::INFO("sent e-mail message to " . $args{'to'}) ;
  return 1 ;
}

# The 'to' and 'cc' attributes can take a single address or a
# comma-delimited list of e-mail addresses.
sub send_email_using_sendmail
{
  my $config = Stanford::Certificate::Config::instance() ;

  my %args = (
                'to'      => undef,
                'cc'      => undef,
                'from'    => $config->{'from_email_address'},
                'subject' => 'NO SUBJECT',
                'body'    => 'NO MESSAGE BODY',
                @_,
             ) ;

  if (!$args{'to'})
  {
    croak "cannot send mail without a To: address" ;
  }

  # Do we override the To: address?
  if ($config->{'to_email_override'})
  {
    # Yes, override.
    $args{'subject'} = $args{'subject'} . " (address " .
                       $args{'to'} . " overridden)" ;
    $args{'to'}      = $config->{'to_email_override'} ;
  }

  # Get the "From:" address
  if (!$args{'from'})
  {
      croak "no From: address; cannot proceed" ;
  }

  # Add the -t option (why??). The -f option ensures that bounces
  # are delivered to $return_address.
  my $return_address = $args{'from'} ;
  my $sendmail = "/usr/sbin/sendmail -t -f '$return_address'" ;

  open(my $SM, "|$sendmail") ;

  print $SM 'To: '   .    $args{'to'}      . "\n" ;

  if ($args{'cc'}) {
      print $SM 'Cc: '   .    $args{'cc'}      . "\n" ;
  }

  print $SM 'From: ' .    $args{'from'}    . "\n" ;
  print $SM 'Subject: ' . $args{'subject'} . "\n" ;
  print $SM "Content-type: text/plain\n\n" ;
  print $SM $args{'body'} ;

  close($SM) ;

  Stanford::Certificate::Log::INFO("sent e-mail message to " . $args{'to'}) ;
  return 1 ;
}


1;
