#<<<
## no critic
use utf8;
package Stanford::Certificate::Result::Certificate;

# Created by DBIx::Class::Schema::Loader
# DO NOT MODIFY THE FIRST PART OF THIS FILE

=head1 NAME

Stanford::Certificate::Result::Certificate

=cut

use strict;
use warnings;

use base 'DBIx::Class::Core';

=head1 TABLE: C<certificates>

=cut

__PACKAGE__->table("certificates");

=head1 ACCESSORS

=head2 id

  data_type: 'integer'
  is_auto_increment: 1
  is_nullable: 0

=head2 version

  data_type: 'integer'
  is_nullable: 0

=head2 subject

  data_type: 'varchar'
  is_nullable: 0
  size: 500

=head2 issuer

  data_type: 'varchar'
  is_nullable: 0
  size: 500

=head2 cn

  data_type: 'varchar'
  is_nullable: 0
  size: 100

=head2 valid_at

  data_type: 'datetime'
  datetime_undef_if_invalid: 1
  is_nullable: 0

=head2 expires_at

  data_type: 'datetime'
  datetime_undef_if_invalid: 1
  is_nullable: 0

=head2 issued_by_id

  data_type: 'integer'
  is_foreign_key: 1
  is_nullable: 1

=head2 pem

  data_type: 'text'
  is_nullable: 0

=head2 serial_number

  data_type: 'varchar'
  is_nullable: 1
  size: 60

=head2 signature_algorithm

  data_type: 'varchar'
  is_nullable: 1
  size: 30

=head2 key_algorithm

  data_type: 'varchar'
  is_nullable: 0
  size: 15

=head2 key_length

  data_type: 'integer'
  is_nullable: 1

=head2 subject_key_identifier

  data_type: 'varchar'
  is_nullable: 1
  size: 60

=head2 authority_key_identifier

  data_type: 'varchar'
  is_nullable: 1
  size: 60

=head2 ca

  data_type: 'tinyint'
  is_nullable: 0

=head2 key_cert_sign

  data_type: 'tinyint'
  is_nullable: 0

=head2 crl_url

  data_type: 'varchar'
  is_nullable: 1
  size: 120

=head2 ocsp_uri

  data_type: 'varchar'
  is_nullable: 1
  size: 120

=head2 contact_email

  data_type: 'varchar'
  is_nullable: 1
  size: 60

=head2 suppress_notifications

  data_type: 'tinyint'
  is_nullable: 0

=head2 created_at

  data_type: 'datetime'
  datetime_undef_if_invalid: 1
  is_nullable: 0

=head2 updated_at

  data_type: 'datetime'
  datetime_undef_if_invalid: 1
  is_nullable: 0

=cut

__PACKAGE__->add_columns(
  "id",
  { data_type => "integer", is_auto_increment => 1, is_nullable => 0 },
  "version",
  { data_type => "integer", is_nullable => 0 },
  "subject",
  { data_type => "varchar", is_nullable => 0, size => 500 },
  "issuer",
  { data_type => "varchar", is_nullable => 0, size => 500 },
  "cn",
  { data_type => "varchar", is_nullable => 0, size => 100 },
  "valid_at",
  {
    data_type => "datetime",
    datetime_undef_if_invalid => 1,
    is_nullable => 0,
  },
  "expires_at",
  {
    data_type => "datetime",
    datetime_undef_if_invalid => 1,
    is_nullable => 0,
  },
  "issued_by_id",
  { data_type => "integer", is_foreign_key => 1, is_nullable => 1 },
  "pem",
  { data_type => "text", is_nullable => 0 },
  "serial_number",
  { data_type => "varchar", is_nullable => 1, size => 60 },
  "signature_algorithm",
  { data_type => "varchar", is_nullable => 1, size => 30 },
  "key_algorithm",
  { data_type => "varchar", is_nullable => 0, size => 15 },
  "key_length",
  { data_type => "integer", is_nullable => 1 },
  "subject_key_identifier",
  { data_type => "varchar", is_nullable => 1, size => 60 },
  "authority_key_identifier",
  { data_type => "varchar", is_nullable => 1, size => 60 },
  "ca",
  { data_type => "tinyint", is_nullable => 0 },
  "key_cert_sign",
  { data_type => "tinyint", is_nullable => 0 },
  "crl_url",
  { data_type => "varchar", is_nullable => 1, size => 120 },
  "ocsp_uri",
  { data_type => "varchar", is_nullable => 1, size => 120 },
  "contact_email",
  { data_type => "varchar", is_nullable => 1, size => 60 },
  "suppress_notifications",
  { data_type => "tinyint", is_nullable => 0 },
  "created_at",
  {
    data_type => "datetime",
    datetime_undef_if_invalid => 1,
    is_nullable => 0,
  },
  "updated_at",
  {
    data_type => "datetime",
    datetime_undef_if_invalid => 1,
    is_nullable => 0,
  },
);

=head1 PRIMARY KEY

=over 4

=item * L</id>

=back

=cut

__PACKAGE__->set_primary_key("id");

=head1 RELATIONS

=head2 certificates

Type: has_many

Related object: L<Stanford::Certificate::Result::Certificate>

=cut

__PACKAGE__->has_many(
  "certificates",
  "Stanford::Certificate::Result::Certificate",
  { "foreign.issued_by_id" => "self.id" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 eventlogs

Type: has_many

Related object: L<Stanford::Certificate::Result::Eventlog>

=cut

__PACKAGE__->has_many(
  "eventlogs",
  "Stanford::Certificate::Result::Eventlog",
  { "foreign.certificate_id" => "self.id" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 incommon_statuses

Type: has_many

Related object: L<Stanford::Certificate::Result::IncommonStatus>

=cut

__PACKAGE__->has_many(
  "incommon_statuses",
  "Stanford::Certificate::Result::IncommonStatus",
  { "foreign.certificate_id" => "self.id" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 issued_by

Type: belongs_to

Related object: L<Stanford::Certificate::Result::Certificate>

=cut

__PACKAGE__->belongs_to(
  "issued_by",
  "Stanford::Certificate::Result::Certificate",
  { id => "issued_by_id" },
  {
    is_deferrable => 1,
    join_type     => "LEFT",
    on_delete     => "RESTRICT",
    on_update     => "RESTRICT",
  },
);

=head2 requests

Type: has_many

Related object: L<Stanford::Certificate::Result::Request>

=cut

__PACKAGE__->has_many(
  "requests",
  "Stanford::Certificate::Result::Request",
  { "foreign.certificate_id" => "self.id" },
  { cascade_copy => 0, cascade_delete => 0 },
);

=head2 sans

Type: has_many

Related object: L<Stanford::Certificate::Result::San>

=cut

__PACKAGE__->has_many(
  "sans",
  "Stanford::Certificate::Result::San",
  { "foreign.certificate_id" => "self.id" },
  { cascade_copy => 0, cascade_delete => 0 },
);

## use critic
#>>>

# Created by DBIx::Class::Schema::Loader v0.07042 @ 2016-04-24 12:24:07
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:Njk0GYVLq0iHlN8nKYY5Sg


# You can replace this text with custom content, and it will be preserved on regeneration

## no critic (Subroutines::RequireArgUnpacking) ;

use Convert::PEM;
use Carp ;
use Data::Dumper ;

use Stanford::Orange::Util qw/ trim tempfile slurp spew run_command_improved/ ;
use Stanford::Certificate::Misc qw/ mysql_time_local mysql_time / ;
use Stanford::Certificate::X509 ;

# Override new method so we can set some defaults.

sub new {
  my ( $class, $attrs ) = @_;

  if (!defined($attrs->{'created_at'})) {
      $attrs->{'created_at'} = mysql_time() ;
  }

  if (!defined($attrs->{'updated_at'})) {
      $attrs->{'updated_at'} = mysql_time() ;
  }

  if (!defined($attrs->{'suppress_notifications'})) {
      $attrs->{'suppress_notifications'} = 0 ;
  }

  my $new = $class->next::method($attrs);

  $new->{'show_email_contact'} = 0 ;

  return $new;
}

sub show_email_contact {
  my ($self) = shift ;
  my ($value) = @_ ;

  if (defined($value)) {
      $self->{'show_email_contact'} = $value ;
  }

  return $self->{'show_email_contact'} ;
}

sub get_schema {
  my ($self) = shift ;
  return $self->result_source->schema ;
}

# Return a brand new empty Certificate object.
sub new_cert
{
  my ($self) = shift ;
  my $schema = $self->get_schema() ;
  return $schema->resultset('Certificate')->new({});
}

sub has_id
{
  my ($self) = shift ;

  if (defined($self->id()) && ($self->id() > 0))
  {
    return 1 ;
  }
  else
  {
    return 0 ;
  }
}

sub internal_action
{
  my ($self) = shift ;

  my ($value) = @_ ;
  if (defined($value))
  {
    $self->{'_internal_action'} = $value ;
  }

  return $self->{'_internal_action'} ;
}

# Let
#  ski     := the subject key identifier
#  aki     := the authority key identifier
#  subject := certificate DN
#  issuer  := issuer DN
#
# A certificate is "self-signed" if
#
#      (subject = issuer)
#  AND (    (one of aki or ski (or both) are empty)
#        OR (aki and ski are both nonempty and equal)
sub is_self_signed
{
  my ($self) = shift ;

  my $subject = $self->subject ;
  my $issuer  = $self->issuer ;
  my $aki     = $self->authority_key_identifier ;
  my $ski     = $self->subject_key_identifier ;

  if (!$subject)
  {
    croak 'a certificate cannot have an empty subject!' ;
  }

  if (!$issuer)
  {
    croak 'a certificate cannot have an empty issuer!' ;
  }

  return (($subject eq $issuer) && (!$aki || !$ski || ($aki eq $ski))) ;
}

sub to_s
{
  my ($self) = shift ;

  my $id                    = $self->id() ;
  my $cn                    = $self->cn() ;
  my $subject               = $self->subject() ;
  my $issuer                = $self->issuer() ;
  my $issued_by_id          = $self->issued_by_id() ;
  my $serial_number         = $self->serial_number() ;
  my $valid_at              = $self->valid_at() ;
  my $expires_at            = $self->expires_at() ;
  my $subject_key_identifier   = $self->subject_key_identifier() ;
  my $authority_key_identifier = $self->authority_key_identifier() ;
  my $key_algorithm         = $self->key_algorithm() ;
  my $key_length            = $self->key_length() ;
  my $created_at            = $self->created_at() ;
  my $updated_at            = $self->updated_at() ;
  my $version               = $self->version() ;
  my $contact_email_certs   = $self->contact_email() ;
  my $signature_algorithm   = $self->signature_algorithm() ;

  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###
  my $format_if_empty_fref = sub
  {
    my ($value) = @_ ;

    my $return_value ;
    if (!defined($value))
    {
      $return_value = '<UNDEFINED>' ;
    }
    else
    {
      $value = trim($value) ;
      if (trim($value) eq q{})
      {
        $return_value = '<EMPTY_STRING>' ;
      }
      else
      {
        $return_value = $value ;
      }
    }
    return $return_value ;
  } ;
  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###  # ## ###

  if ($expires_at)
  {
    $expires_at = mysql_time_local($expires_at) ;
  }

  $id            = $format_if_empty_fref->($id) ;
  $cn            = $format_if_empty_fref->($cn) ;
  $subject       = $format_if_empty_fref->($subject) ;
  $issuer        = $format_if_empty_fref->($issuer) ;
  $issued_by_id  = $format_if_empty_fref->($issued_by_id) ;
  $serial_number = $format_if_empty_fref->($serial_number) ;
  $valid_at      = $format_if_empty_fref->($valid_at) ;
  $expires_at    = $format_if_empty_fref->($expires_at) ;
  $subject_key_identifier   = $format_if_empty_fref->($subject_key_identifier) ;
  $authority_key_identifier = $format_if_empty_fref->($authority_key_identifier) ;
  $key_algorithm = $format_if_empty_fref->($key_algorithm) ;
  $key_length    = $format_if_empty_fref->($key_length) ;
  $version       = $format_if_empty_fref->($version) ;
  $contact_email_certs = $format_if_empty_fref->($contact_email_certs) ;
  $signature_algorithm = $format_if_empty_fref->($signature_algorithm) ;

  ## CA
  my $ca_formatted ;
  if ($self->ca())
  {
    $ca_formatted = 'YES' ;
  }
  else
  {
    $ca_formatted = 'NO' ;
  }

  ## sans (subject alternative names)
  my @sans_formatted_lines = () ;
  my @sans = $self->sans()->all() ;
  foreach my $san (@sans)
  {
   my $current_san_formatted = q{ } . $san->san_type()->name() . ': ' . $san->name() ;
   push(@sans_formatted_lines, $current_san_formatted) ;
  }

  my $sans_formatted ;
  if (!@sans_formatted_lines)
  {
    $sans_formatted = ' <No Subject Alternative Names>' ;
  }
  else
  {
    $sans_formatted = join("\n", @sans_formatted_lines) ;
  }

  # Find the certreq object pointing at this certificate.
  my $schema = $self->result_source->schema ;
  my @reqs = $schema->resultset('Request')->search(
        { certificate_id => $id },
        );

  my $contact_info ;
  if (scalar(@reqs) == 0) {
      $contact_info = 'NONE' ;
  } elsif (scalar(@reqs) == 1) {
      my $req = $reqs[0] ;
      my $contact_email = $req->contact_email() ;
      my $suppressed    = $req->suppress_notifications() ;
      if ($suppressed) {
          $contact_info = "$contact_email (suppressed)" ;
      } else {
          $contact_info = $contact_email ;
      }
  } else {
      croak 'found ' . scalar(@reqs) . ' certificate requests ' .
            "associated with certificate $id (should be no more than 1)" ;
  }

  my @incommon_statuses = $self->incommon_statuses() ;
  @incommon_statuses =
      reverse Stanford::Certificate::Result::IncommonStatus::sort_statuses(\@incommon_statuses) ;

  my @statuses = () ;
  foreach my $incommon_status (@incommon_statuses) {
      my $status    = $incommon_status->status() ;
      my $status_at = $incommon_status->status_at() ;
      if (!$status_at) {
          $status_at = 'NO DATE' ;
      }
      push(@statuses, sprintf('  %-10s [%s]', $status,$status_at)) ;
  }
  my $statuses ;
  if (scalar(@statuses) == 0) {
      $statuses = '  NO STATUSES' ;
  } else {
      $statuses = join("\n", @statuses) ;
  }

  return <<"EOS";
id:                        $id
cn (common name):          $cn
subject:                   $subject
subject alternative names:
$sans_formatted
issuer:                    $issuer
issuer id:                 $issued_by_id
CA?:                       $ca_formatted
contact (request):         $contact_info
contact (cert table):      $contact_email_certs
serial_number:             $serial_number
valid_at:                  $valid_at
expires_at:                $expires_at
subject_key_identifier:    $subject_key_identifier
authority_key_identifier:  $authority_key_identifier
signature algorithm:       $signature_algorithm
key_algorithm:             $key_algorithm
key_length:                $key_length
version:                   $version
created_at:                $created_at
updated_at:                $updated_at
statuses:
$statuses
EOS
}

# Looks for a certificate in the database that matches this one.
# Uses (serial number, issuer) as key. Returns a result set of Certificate
# objects.
sub readx
{
  my ($self) = shift ;

  # Make sure serial number is a string!
  my $serial_number = $self->serial_number ;

  if (!defined($serial_number))
  {
    croak 'cannot call readx on a certificate with an undefined serial number' ;
  }

  $serial_number = q{} . $self->serial_number ;
  if ($serial_number eq q{})
  {
    croak 'cannot call readx on a certificate with an empty serial number' ;
  }

  my $issuer = $self->issuer() ;
  if (!$issuer)
  {
    croak 'cannot call readx on a certificate without an issuer' ;
  }

  my $schema = $self->result_source->schema ;

  my $certificates_rs = $schema->resultset('Certificate')->search(
        { serial_number => $serial_number, issuer => $issuer },
      );

  return $certificates_rs ;
}

# Call the read method but only return one certificate. If more than one
# certificate is found, die. If no certificates found, return nothing.
# Returns undef if no certificates found.
sub read_one
{
  my ($self) = shift ;

  my $certificates_rs = $self->readx() ;
  my $num_certs = $certificates_rs->count() ;

  my $serial_number = $self->serial_number();
  my $issuer        = $self->issuer();

  if ($num_certs == 0)
  {
    return ;
  }
  elsif ($num_certs > 1)
  {
    croak 'found more than one certificate with (serial number, issuer) = ' .
        "('$serial_number', '$issuer')" ;
  }
  else
  {
    return $certificates_rs->next() ;
  }
}

# Returns undef if no certificates found.
sub find_by_id
{
  my ($self) = shift ;

  my ($id) = @_ ;

  my $schema = $self->result_source->schema ;

  my $certs = $schema->resultset('Certificate')->search(
        { id => $id },
  );

  if ($certs)
  {
    my @all_certs = $certs->all() ;
    if (@all_certs)
    {
      if (scalar(@all_certs) > 1)
      {
        croak "found more than one certificate with id '$id'!!)" ;
      }
      else
      {
        my $cert = $all_certs[0] ;
        return $cert ;
      }
    }
  }
  return ;
}

# Returns undef if no certificates found. Returns a single certificate if found.
sub find_by_order_number
{
  my ($self) = shift ;

  my ($order_number) = @_ ;

  my $schema  = $self->get_schema() ;
  my $request = $schema->resultset('Request')->new({});

  my $request_found = $request->findByOrderNumber($order_number) ;
  if (!defined($request_found)) {
      return ;
  }

  my $certificate_id = $request_found->certificate_id() ;

  my $cert = $schema->resultset('Certificate')->new({}) ;

  return $cert->find_by_id($certificate_id) ;
}

# Returns the empty array if no certificates found.
sub find_by_cn
{
  my ($self) = shift ;

  my ($cn) = @_ ;

  my $schema = $self->result_source->schema ;

  my $certs = $schema->resultset('Certificate')->search(
        { cn => $cn },
  );

  if ($certs)
  {
    return $certs->all() ;
  }
  else
  {
    return () ;
  }
}

# Returns an array of all the certificates that have the provided
# subject. So, the returned array could be empty (no certificates match
# the provided subject), a singleton (only one certificate has a matching
# subject), or contain more than one element (more than one certificate
# has a matching subject).
sub find_by_subject
{
  my ($self) = shift ;

  my ($subject, $schema) = @_ ;

  if (!$schema)
  {
    $schema = $self->result_source->schema ;
  }

  my $certs = $schema->resultset('Certificate')->search(
        { subject => $subject },
  );

  if ($certs)
  {
    return $certs->all() ;
  }
  else
  {
    return ();
  }
}

# Returns an array of Certificate objects (array could be empty).
sub find_by_serial_number_and_issuer
{
  my ($self) = shift ;

  my ($serial_number, $issuer, $schema) = @_ ;

  if (!$schema)
  {
    $schema = $self->result_source->schema ;
  }

  my $certificates_rs = $schema->resultset('Certificate')->search(
        { serial_number => $serial_number, issuer => $issuer },
      );

  if ($certificates_rs)
  {
    return $certificates_rs->all() ;
  }
  else
  {
    return ();
  }
}

# Returns undef if no certificates found.
sub find_issuer_certificate
{
  my ($self) = shift ;

  my $schema = $self->result_source->schema ;

  # 1. Try finding it via the authority key identifier.
  my $authority_key_identifier = $self->authority_key_identifier() ;
  if ($authority_key_identifier)
  {
    my $issuers = $schema->resultset('Certificate')->search(
          { subject_key_identifier => $authority_key_identifier },
      );

    if ($issuers)
    {
      my @all_issuers = $issuers->all() ;
      if (@all_issuers)
      {
        if (scalar(@all_issuers) > 1) {
            my $id = $self->id() ;
            croak "found more than one issuer for $id (authority key id: $authority_key_identifier!!)" ;
        } else {
          my $issuer_cert = $all_issuers[0] ;
          return $issuer_cert ;
        }
      }
    }
    else
    {
      carp 'no issuer found for ' . $self->cn() ;
    }
  }

  # 2. Try finding it via issuer field.
  my $issuer = $self->issuer() ;
  if ($issuer)
  {
    my $issuers = $schema->resultset('Certificate')->search(
          { subject => $issuer },
      );

    if ($issuers)
    {
      my @all_issuers = $issuers->all() ;
      if (@all_issuers)
      {
        if (scalar(@all_issuers) > 1)
        {
          croak "found more than one issuer (issuer subject: $issuer!!)" ;
        }
        else
        {
          my $issuer_cert = $all_issuers[0] ;
          return $issuer_cert ;
        }
      }
    }
  }
  else
  {
    croak 'this certificate does not have an issuer!' ;
  }
}

sub derive_cn
{
  my ($self) = shift ;

  my $subject = $self->subject ;

  if (!$subject)
  {
    return ;
  }
  else
  {
    if ($subject =~ m{^.*CN=(.*)\s*$}xsm)
    {
        $self->cn($1) ;
        return $self->cn ;
    }
    else
    {
        return ;
    }
  }
}


# Request the certificate (uses the incommon-mgr script).
sub request
{
  my ($self) = shift ;

  my ($username, $password) = @_ ;

  # Get the parameters needed.
  my $server_type = $self->server_type() ;
  my $term = $self->duration ;
  my $server_type_name = $server_type->name() ;

  # Create a temporary file with the contents of the CSR.
  my $tempfile = tempfile() ;
  spew($self->csr(), $tempfile) ;

  my @cmd = (
      'incommon-mgr', 'request',
      $tempfile,
      '--username',    $username,
      '--password',    $password,
      '--term',        $term,
      '--server-type', $server_type_name,
      ) ;

  if ($self->test) {
      push(@cmd, '--test') ;
  }

  my ($stdout, $stderr, $rc) = run_command_improved(@cmd) ;
  $stdout = trim($stdout) ;

  if (!$stdout || ($stdout !~ m{^\d+$}xsm))
  {
    croak 'request did not return an order number' ;
  }
  else
  {
    $self->order_number($stdout) ;
    $self->update() ;
  }

   unlink $tempfile
    or croak "could not unlink file '$tempfile'" ;

  return $self->order_number ;
}

# Request the certificate (uses the incommon-mgr script).
sub collect
{
  my ($self) = shift ;

  my ($order_number, $username, $password, $test) = @_ ;

  if (!$order_number)
  {
    croak 'no order number!?!' ;
  }

  $self->derive_cn() ;
  if (!$self->cn())
  {
    croak 'cannot collect with an undefined CN' ;
  }

  # Get the cert using incommon-mgr
  # Get the parameters needed.
  my @cmd = (
      'incommon-mgr', 'collect',
      $order_number,
      '--username', $username,
      '--password', $password,
      ) ;

  if ($self->test()) {
      push(@cmd, '--test') ;
  }

  my ($stdout, $stderr, $rc) = run_command_improved(@cmd) ;
  $stdout = trim($stdout) ;

  my @certs ;
  if (!$stdout) {
    croak 'collect did not return certificate bundle' ;
  } elsif ($stdout !~ m{-----BEGIN\ CERTIFICATE-----.*-----END\ CERTIFICATE-----.*}xms) {
    croak 'collect returned something that does not look like a certificate bundle' ;
  } else {
    @certs = $self->parse_pem_string($stdout) ;
  }

  return @certs ;
}

# Revoke the certificate (uses the incommon-mgr script).
sub revoke {
  my ($self) = shift ;

  if ($self->is_revoked()) {
      croak 'this certificate is already revoked' ;
  }

  my ($credentials_file, $test, $reason) = @_ ;

  if (!$reason) {
    croak 'missing revocation reason!' ;
  }

  my $id = $self->id() ;

  # Find the request associated with this certificate.
  my $schema  = $self->get_schema() ;
  my $request = $schema->resultset('Request')->new({});

  my $request1 = $request->findByCertificateId($id) ;
  if (!$request1) {
      croak 'no request found for this certificate' ;
  }

  my $order_number = $request1->order_number() ;

  if (!$order_number) {
    croak 'request has no order number!?!' ;
  }


  my @cmd = ('incommon-mgr',
             'revoke',
             $order_number,
             $reason,
             '--credentials-file', $credentials_file,
      ) ;

  if ($test) {
    push(@cmd, '--test') ;
  }

  my ($stdout, $stderr, $exit_code) = run_command_improved(@cmd) ;
  $stdout = trim($stdout) ;

  if ($stderr) {
      croak "cert revoke failed: $stderr" ;
  } elsif ($stdout =~ m{succeeded}xsm) {
      return 1 ;
  } else {
      croak "cert revoke failed [$stdout]" ;
  }
}




sub get_subject_from_pem
{
  my ($pem_string) = @_ ;
  return ;
}

# Takes as its argument a file of certificate PEM strings.
#
# Return either a single cert or an array of certs, depending on the
# context this function was called in. Note that if this is called in
# scalar context but there are multiple certs, we return just the first
# one.
#
# If the $force argument is true, will create the certificate even if
# there is already a certificate in the database with the same issuer and
# serial number.
sub make_cert_from_pem_file
{
  my ($self) = shift ;

  my ($cert_pem_file, $force) = @_ ;

  # Convert to a string.
  my $pem_string = slurp($cert_pem_file) ;

  # In case the string has multiple certificates, split into individual
  # certificate strings
  my @pem_strings = Stanford::Certificate::X509::split_pems_string($pem_string) ;

  my @certs = $self->make_certs_from_pem_string(\@pem_strings, $force) ;

  if (wantarray())
  {
    return @certs ;
  }
  else
  {
    return $certs[0] ;
  }
}

# Array version of make_cert_from_pem_string. Expects array to be passed by
# reference.
#
# If $force is true, will add to database even if there is already a
# certificate in the database with the same issuer and serial number.
#
# Returns an array of Certificate objects.
sub make_certs_from_pem_string
{
  my ($self) = shift ;

  my ($cert_string_pems_aref, $force) = @_ ;

  if (ref($cert_string_pems_aref) ne 'ARRAY')
  {
    croak 'expecting an array reference' ;
  }

  my @cert_string_pems = @{ $cert_string_pems_aref };

  my @certs = () ;
  foreach my $cert_string_pem (@cert_string_pems)
  {
    my $new_cert = $self->make_cert_from_pem_string($cert_string_pem, $force) ;
    push(@certs, $new_cert) ;
  }

  return @certs ;
}


# Given a PEM string for a certificate, create a certificate object,
# save it in the database, and return the new certificate object.
#
# As a side effect, save the action in _internal_action:
#  + 'read'
#  + 'inserted'
#  + 'force_inserted'
#
# If there is an object in the database with the same issuer and serial
# number, update $self with that object (unless $force is true) and return
# 'read'. If $force is true, create a new database entry and return
# 'force_insert'.
sub make_cert_from_pem_string {
  my ($self) = shift ;

  my ($cert_string_pem, $force) = @_ ;

  my $schema = $self->result_source->schema ;

  my $cert = $self->new_cert() ;

  # Populate this new certificate object's attributes from the PEM string:
  my %pem_field_to_value = Stanford::Certificate::X509::parse_x509_pem($cert_string_pem) ;
  $cert->populate_attributes($cert_string_pem, \%pem_field_to_value) ;
  my $serial_number = $cert->serial_number() ;
  my $issuer        = $cert->issuer() ;

  # If the issuer or serial number is not defined, this is a big problem.
  if (!defined($serial_number))
  {
    croak 'cannot make a cert without a serial number!' ;
  }

  if (!$issuer)
  {
    croak 'cannot make a cert without an issuer!' ;
  }

  # Does this certificate already exist in the database? If so, do not
  # save it but, rather, return the existing certificate.
  my @found_certs = $self->find_by_serial_number_and_issuer($serial_number, $issuer) ;
  my $number_matching_certs = scalar(@found_certs) ;

  my $internal_action ;
  if ($number_matching_certs > 0)
  {
    if (!$force && ($number_matching_certs > 1))
    {
      my $msg = 'the database has multiple certificates with (serial number, issuer) = ' .
                "($serial_number, $issuer)" ;
      $self->log_info($msg) ;
      croak $msg ;
    }
    else
    {
      # If we get here then we know there is already exactly one
      # certificate already in the database with the same issuer and
      # serial number. Unless we are forcing the add, do not add but
      # replace $self with the object in the database.
      if (!$force)
      {
        # Since we are not forcing, update $self with the existing certificate.
        $self->log_info('The database already has a certificate with (serial number, issuer) = ' .
                            "($serial_number, $issuer) so skipping add.") ;
        my $found_cert = $found_certs[0] ;
        $found_cert->internal_action('read') ;
        return $found_cert ;
      }
      else
      {
        # There is already a matching record but we add anyway.
        $internal_action = 'force_inserted' ;
      }
    }
  }
  else
  {
    # no certificate in database, so we will insert.
    $internal_action = 'inserted' ;
  }

  # Before we do the sans (subject alternative names), save this object.
  $cert->insert() ;

  # Refresh by re-reading the just-inserted record (will populate the primary key):
  $cert->discard_changes() ;
  $cert->internal_action($internal_action) ;
  my $cert_id = $cert->id() ;
  $cert->log_info('inserted new certificate (serial number, issuer) = ' .
                  "($serial_number, $issuer) into database (id=$cert_id).") ;

  # See if the issuer of this certificate is in the database.
  my $issuer_cert = $cert->find_issuer_certificate() ;
  if ($issuer_cert)
  {
    $cert->issued_by_id($issuer_cert->id()) ;
    $cert->update() ;
  }

  ## Subject Alternative Names
  my $sans_href = $pem_field_to_value{'sans'} ;
  if ($sans_href)
  {
    my %sans = %{ $sans_href } ;
    my @san_types = keys %sans ;

    foreach my $san_type (@san_types)
    {
      my $new_san_types = $schema->resultset('SanType')->search(
        { name => $san_type },
      );

      my $new_san_type = $new_san_types->next() ;
      if (!$new_san_type)
      {
        croak "could not find a SanType with name '$san_type'" ;
      }

      my @sans_of_this_type = @{ $sans{$san_type} } ;
      foreach my $san_of_this_type (@sans_of_this_type)
      {
        my $new_san = $schema->resultset('San')->new({});
        $new_san->san_type_id($new_san_type->id()) ;
        $new_san->name($san_of_this_type) ;
        $new_san->certificate_id($cert_id) ;
        $new_san->insert() ;
      }
    }
  }

  if (!defined($cert->internal_action()))
  {
    croak 'no internal action set?!?' ;
  }
  return $cert ;
}


## Does NOT insert into datbase.
#sub new_cert_from_pem_string
#{
#  my ($self) = shift ;
#
#  my ($cert_string_pem, $force) = @_ ;
#
#  my $schema = $self->result_source->schema ;
#
#  # Populate this new certificate object's attributes from the PEM string:
#  my %pem_field_to_value = Stanford::Certificate::Misc::parse_x509_pem($cert_string_pem) ;
#  my $new_cert = $self->populate_cert_attributes_from_pem_string($cert_string_pem,
#                                                                 \%pem_field_to_value) ;
#  my $serial_number = $new_cert->serial_number() ;
#  my $issuer        = $new_cert->issuer() ;
#
#  # See if the issuer of this certificate is in the database.
#  my $issuer_cert = $new_cert->find_issuer_certificate() ;
#  if ($issuer_cert)
#  {
#    $new_cert->issued_by_id($issuer_cert->id()) ;
#  }
#
#  ## Subject Alternative Names
#  my $sans_href = $pem_field_to_value{'sans'} ;
#  if ($sans_href)
#  {
#    my %sans = %$sans_href ;
#    my @san_types = keys %sans ;
#
#    foreach my $san_type (@san_types)
#    {
#      my $new_san_types = $schema->resultset('SanType')->search(
#        { name => $san_type },
#      );
#
#      my $new_san_type = $new_san_types->next() ;
#      if (!$new_san_type)
#      {
#        croak "could not find a SanType with name '$san_type'" ;
#      }
#
#      my @sans_of_this_type = @{ $sans{$san_type} } ;
#      foreach my $san_of_this_type (@sans_of_this_type)
#      {
#        my $new_san = $new_cert->new_related('sans',
#                         {
#                           'san_type_id' => $new_san_type->id(),
#                           'name'        => $san_of_this_type,
#                         }
#                      ) ;
#      }
#    }
#  }
#  else
#  {
#    # No SANs to add.
#  }
#
#  return $new_cert ;
#}

# Set the attributes of the current Certificate object with the values
# from the PEM string or the hash. Note that only those attributes
# derivable from one of those sources are set; other existing Certificate
# attributes will be unchanged (e.g., test).
#
# We get the values either from the first argument (a PEM string)
# or from the second argument (a hash mapping attribute name to value);
# the second argument takes precedence.
#
# Note that the id attribute will, of course, NOT be set as we have not
# inserted or read from the database. Also, the related SAN information is
# not populated.
#
# Returns: nothing.
sub populate_attributes
{
  my ($self) = shift ;

  my ($cert_string_pem, $pem_field_to_value_href) = @_ ;

  my %pem_field_to_value ;
  if (!defined($pem_field_to_value_href))
  {
    %pem_field_to_value = Stanford::Certificate::Misc::parse_x509_pem($cert_string_pem) ;
  }
  else
  {
    %pem_field_to_value = %{ $pem_field_to_value_href };
  }

  $self->version($pem_field_to_value{'version'}) ;
  $self->subject($pem_field_to_value{'subject'}) ;
  $self->issuer($pem_field_to_value{'issuer'}) ;
  $self->cn($pem_field_to_value{'cn'}) ;
  $self->valid_at($pem_field_to_value{'valid_at'}) ;
  $self->expires_at($pem_field_to_value{'expires_at'}) ;
  $self->key_length($pem_field_to_value{'key_length'}) ;
  $self->key_algorithm($pem_field_to_value{'key_algorithm'}) ;
  $self->serial_number($pem_field_to_value{'serial_number'}) ;
  $self->subject_key_identifier($pem_field_to_value{'subject_key_identifier'}) ;
  $self->authority_key_identifier($pem_field_to_value{'authority_key_identifier'}) ;
  $self->ocsp_uri($pem_field_to_value{'ocsp_uri'}) ;
  $self->pem($cert_string_pem) ;
  $self->ca($pem_field_to_value{'ca'}) ;
  $self->signature_algorithm($pem_field_to_value{'signature_algorithm'}) ;

  my $serial_number = $self->serial_number() ;
  my $issuer        = $self->issuer() ;

  # Set the key_cert_sign field
  my $key_usage_href = $pem_field_to_value{'key_usage_href'} ;
  my %key_usage = %{ $key_usage_href } ;
  if (exists($key_usage{'keyCertSign'}))
  {
    $self->key_cert_sign(1) ;
  }
  else
  {
    $self->key_cert_sign(0) ;
  }

  return ;
}

# Verify that the the chain of signers all exist in the database for this
# certificate.
#
# Returns 1 on success, 0 on failure.
sub check_signing_chain
{
  my ($self) = shift ;

  my $cert = $self->read_one() ;

  # depth protections
  my $max_depth = 20 ;
  my $counter = 0 ;
  while (!$cert->is_self_signed())
  {
    ++$counter ;
    if ($counter > $max_depth)
    {
      carp $cert->to_s() ;
      croak "exceeded max depth of $max_depth verifying signing chain" ;
    }

    my $issued_by_id = $cert->issued_by_id() ;
    if (!$issued_by_id)
    {
      # This certificate does not have an issuer, so return false.
      return 0 ;
    }

    my $issuer_cert = $cert->find_by_id($issued_by_id) ;

    if (!$issuer_cert)
    {
      my $id = $cert->id() ;
      return 0 ;
    }
    else
    {
      $cert = $cert->find_by_id($issued_by_id) ;
    }
  }
  return 1 ;
}

sub log_message
{
  my $self = shift ;

  my ($level, $message) = @_ ;

  my $schema = $self->result_source->schema ;

  my $new_eventlog = $schema->resultset('Eventlog')->new({});
  $new_eventlog->level($level) ;
  $new_eventlog->message($message) ;
  $new_eventlog->certificate_id($self->id()) ;

  $new_eventlog->insert() ;
  $new_eventlog->discard_changes() ;

  return $new_eventlog ;
}

sub log_info
{
  my ($self) = shift ;

  my ($message) = @_ ;

  my %text_to_number = Stanford::Certificate::Result::Eventlog::text_to_number() ;

  return $self->log_message($text_to_number{'INFO'}, $message) ;
}

sub get_all_events
{
  my ($self) = shift ;

  my $schema = $self->result_source->schema ;
  my $eventlogs_rs = $schema->resultset('Eventlog')->search(
        { certificate_id => $self->id() },
      );

  return $eventlogs_rs ;
}

sub events_formatted
{
  my ($self) = shift ;

  my $eventlogs_rs = $self->get_all_events() ;
  return Stanford::Certificate::Result::Eventlog::format_eventlog_rs($eventlogs_rs) ;
}

# Returns a ResultSet object.
sub find_cert
{
  my ($self) = shift ;

  my ($substring) = @_ ;

  if (!$substring)
  {
    croak q{cannot call method 'find' on an empty string} ;
  }

  my $schema = $self->result_source->schema ;
  my $certs_rs = $schema->resultset('Certificate')->search(
        { subject => { like => q{%} . $substring . q{%}} },
      );

  return $certs_rs ;
}

sub format_as_cert_for_list {
    my $self = shift ;

    my ($options_href) = @_ ;
    my %options ;
    if ($options_href) {
        %options = %{ $options_href } ;
    } else {
        %options = () ;
    }

    my $id            = $self->id() ;
    my $cn            = $self->cn() ;
    my $expires_at    = $self->expires_at() ;
    my $contact_email = $self->contact_email() ;

    # Remove the time portion of $expires_at.
    $expires_at =~ s{\s+\d\d:\d\d:\d\d$}{}xsm ;

    my $signing_cert ;
    if ($self->key_cert_sign()) {
        $signing_cert = '[S]' ;
    } else {
        $signing_cert = q{} ;
    }

    if ($self->show_email_contact()) {
        if (!$contact_email) {
            $contact_email = 'UNDEFINED' ;
        }
        return sprintf("%7u %s %s%s <%s>\n", $id, $expires_at, $signing_cert,
        $cn, $contact_email) ;
    } else {
        return sprintf("%7u %s %s%s\n", $id, $expires_at, $signing_cert, $cn) ;
    }
}

sub format_list_header {
    my ($self) = shift ;

    if ($self->show_email_contact()) {
        return sprintf("%7s %-10s %s\n", 'id', 'expires', 'cname <contact-email>') ;
    } else {
        return sprintf("%7s %-10s %s\n", 'id', 'expires', 'cname') ;
    }
}

# Returns the lowest-numbered incommon_status object. If no
# incommon_status objects, returns undef.
sub status {
    my ($self) = shift ;
    my @incommon_statuses = $self->incommon_statuses() ;

    if (scalar(@incommon_statuses) == 0) {
        return ;
    } else {
        my @sorted =
          Stanford::Certificate::Result::IncommonStatus::sort_statuses(\@incommon_statuses) ;
        return $sorted[0] ;
    }
}

sub has_status {
    my ($self) = shift ;
    my ($status) = @_ ;

    my @incommon_statuses = $self->incommon_statuses() ;

    foreach my $incommon_status (@incommon_statuses) {
        if ($status eq $incommon_status->status()) {
            return 1 ;
        }
    }
    return 0 ;
}

# Be sure not to add if there is already one of that type. Returns
# 1 if the new incommon status does not already have one of that type,
# 0 otherwise.
sub add_status {
    my ($self) = shift ;
    my ($incommon_status) = @_ ;

    if ($self->has_status($incommon_status->status())) {
        # Already have it, so do not add.
        return 0 ;
    } else {
        my $values_href = {
            'status'    => $incommon_status->status(),
            'status_at' => $incommon_status->status_at(),
        } ;
        eval {
            $self->add_to_incommon_statuses($values_href) ;
        } ;
        my $eval_error = $@ ;
        if ($eval_error) {
            croak "failed to add incommon status: $@" ;
        }
        return 1 ;
    }
}

sub is_expired {
    my ($self) = shift ;

    my $expires_at = $self->expires_at() ;
    if (!$expires_at) {
        croak 'this certificate does not have an expiration?!' ;
    }

    if ($expires_at lt mysql_time()) {
        return 1 ;
    } else {
        return 0 ;
    }
}

sub is_revoked {
    my ($self) = shift ;

    my @incommon_statuses = $self->incommon_statuses() ;

    foreach my $incommon_status (@incommon_statuses) {
        if ($incommon_status->is_revoked()) {
            return 1 ;
        }
    }

    # If we get here, none of the incommon statuses were revoked.
    return 0 ;
}

sub find_and_set_issuer {
    my ($self) = shift ;

    my $issuer_cert = $self->find_issuer_certificate() ;

    if (!$issuer_cert) {
        my $id = $self->id() ;
        croak "could not find issuer certificate for $id" ;
    }

    $self->issued_by_id($issuer_cert->id()) ;
    $self->update() ;

    return $issuer_cert ;
}

1;

__END__
