#<<<
## no critic
use utf8;
package Stanford::Certificate::Result::Eventlog;

# Created by DBIx::Class::Schema::Loader
# DO NOT MODIFY THE FIRST PART OF THIS FILE

=head1 NAME

Stanford::Certificate::Result::Eventlog

=cut

use strict;
use warnings;

use base 'DBIx::Class::Core';

=head1 TABLE: C<eventlog>

=cut

__PACKAGE__->table("eventlog");

=head1 ACCESSORS

=head2 id

  data_type: 'integer'
  is_auto_increment: 1
  is_nullable: 0

=head2 request_id

  data_type: 'integer'
  is_foreign_key: 1
  is_nullable: 1

=head2 certificate_id

  data_type: 'integer'
  is_foreign_key: 1
  is_nullable: 1

=head2 level

  data_type: 'integer'
  is_nullable: 0

=head2 message

  data_type: 'text'
  is_nullable: 0

=head2 created_at

  data_type: 'datetime'
  datetime_undef_if_invalid: 1
  is_nullable: 0

=cut

__PACKAGE__->add_columns(
  "id",
  { data_type => "integer", is_auto_increment => 1, is_nullable => 0 },
  "request_id",
  { data_type => "integer", is_foreign_key => 1, is_nullable => 1 },
  "certificate_id",
  { data_type => "integer", is_foreign_key => 1, is_nullable => 1 },
  "level",
  { data_type => "integer", is_nullable => 0 },
  "message",
  { data_type => "text", is_nullable => 0 },
  "created_at",
  {
    data_type => "datetime",
    datetime_undef_if_invalid => 1,
    is_nullable => 0,
  },
);

=head1 PRIMARY KEY

=over 4

=item * L</id>

=back

=cut

__PACKAGE__->set_primary_key("id");

=head1 RELATIONS

=head2 certificate

Type: belongs_to

Related object: L<Stanford::Certificate::Result::Certificate>

=cut

__PACKAGE__->belongs_to(
  "certificate",
  "Stanford::Certificate::Result::Certificate",
  { id => "certificate_id" },
  {
    is_deferrable => 1,
    join_type     => "LEFT",
    on_delete     => "RESTRICT",
    on_update     => "RESTRICT",
  },
);

=head2 request

Type: belongs_to

Related object: L<Stanford::Certificate::Result::Request>

=cut

__PACKAGE__->belongs_to(
  "request",
  "Stanford::Certificate::Result::Request",
  { id => "request_id" },
  {
    is_deferrable => 1,
    join_type     => "LEFT",
    on_delete     => "RESTRICT",
    on_update     => "RESTRICT",
  },
);

## use critic
#>>>

# Created by DBIx::Class::Schema::Loader v0.07042 @ 2016-04-24 12:24:07
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:TC97NdpwDsM14TORg48tyA

# You can replace this text with custom content, and it will be preserved on regeneration

use Stanford::Certificate::Misc qw(mysql_time) ;

# Mapping of numbers to level text
our %NUMBER_TO_TEXT =
(
  1 => 'DEBUG',
  2 => 'INFO',
  3 => 'WARN',
  4 => 'ERROR',
) ;

my %TEXT_TO_NUMBER = reverse %NUMBER_TO_TEXT    ;

sub text_to_number {
    return %TEXT_TO_NUMBER ;
}

sub new
{
  my ( $class, $attrs ) = @_;

  $attrs->{'level'} = $TEXT_TO_NUMBER{'INFO'} ;
  $attrs->{'created_at'} = mysql_time() unless defined $attrs->{'created_at'};

  my $new = $class->next::method($attrs);
  return $new;
}

# Takes a result set and format it as a sequence string, one line for each
# eventlog.
sub format_eventlog_rs
{
  my ($eventlogs_rs) = @_ ;

  my $rv = q{} ;
  while (my $eventlog = $eventlogs_rs->next())
  {
    my $created_at = $eventlog->created_at() ;
    my $message    = $eventlog->message() ;
    my $level      = $eventlog->level() ;
    my $level_text = lc($NUMBER_TO_TEXT{$level}) ;
    $rv .=  "$created_at [$level_text] $message\n" ;
  }

  return $rv ;
}


1;
