#<<<
## no critic
use utf8;
package Stanford::Certificate::Result::LegacyExpiration;

# Created by DBIx::Class::Schema::Loader
# DO NOT MODIFY THE FIRST PART OF THIS FILE

=head1 NAME

Stanford::Certificate::Result::LegacyExpiration

=cut

use strict;
use warnings;

use base 'DBIx::Class::Core';

=head1 TABLE: C<legacy_expirations>

=cut

__PACKAGE__->table("legacy_expirations");

=head1 ACCESSORS

=head2 id

  data_type: 'integer'
  is_auto_increment: 1
  is_nullable: 0

=head2 contact_email

  data_type: 'varchar'
  is_nullable: 0
  size: 60

=head2 cn

  data_type: 'varchar'
  is_nullable: 0
  size: 60

=head2 expires_at

  data_type: 'date'
  datetime_undef_if_invalid: 1
  is_nullable: 0

=head2 suppress_notifications

  data_type: 'tinyint'
  is_nullable: 0

=head2 created_at

  data_type: 'datetime'
  datetime_undef_if_invalid: 1
  is_nullable: 0

=head2 updated_at

  data_type: 'datetime'
  datetime_undef_if_invalid: 1
  is_nullable: 0

=cut

__PACKAGE__->add_columns(
  "id",
  { data_type => "integer", is_auto_increment => 1, is_nullable => 0 },
  "contact_email",
  { data_type => "varchar", is_nullable => 0, size => 60 },
  "cn",
  { data_type => "varchar", is_nullable => 0, size => 60 },
  "expires_at",
  { data_type => "date", datetime_undef_if_invalid => 1, is_nullable => 0 },
  "suppress_notifications",
  { data_type => "tinyint", is_nullable => 0 },
  "created_at",
  {
    data_type => "datetime",
    datetime_undef_if_invalid => 1,
    is_nullable => 0,
  },
  "updated_at",
  {
    data_type => "datetime",
    datetime_undef_if_invalid => 1,
    is_nullable => 0,
  },
);

=head1 PRIMARY KEY

=over 4

=item * L</id>

=back

=cut

__PACKAGE__->set_primary_key("id");

## use critic
#>>>

# Created by DBIx::Class::Schema::Loader v0.07042 @ 2016-04-24 12:24:07
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:9o13uj8t9Z9aaCPXYURPeA


# You can replace this text with custom content, and it will be preserved on regeneration

# Override new method so we can set some defaults.

use Stanford::Certificate::Misc qw/ mysql_time / ;
use Carp ;

sub new
{
  my ( $class, $attrs ) = @_;

  $attrs->{'created_at'} = mysql_time() unless defined $attrs->{'created_at'};
  $attrs->{'updated_at'} = mysql_time() unless defined $attrs->{'updated_at'};
  $attrs->{'suppress_notifications'} = 0 unless defined $attrs->{'suppress_notifications'};

  my $new = $class->next::method($attrs);
  return $new;
}

sub make_fake
{
  my ($schema, $values_href) = @_ ;

  my $le = $schema->resultset('LegacyExpiration')->new({});

  $le->contact_email('leland.test@mailtonews.stanford.edu') ;
  $le->expires_at('2012-12-25') ;
  $le->cn('adamtestabc.stanford.edu') ;

  return $le ;
}

sub to_s {
    my $self = shift ;

    my $suppress = q{} ;
    if ($self->suppress_notifications()) {
        $suppress = ' (suppressed)' ;
    }

    my $expires_at    = $self->expires_at() ;
    my $cn            = $self->cn() ;
    my $contact_email = $self->contact_email() ;
    my $id            = $self->id() ;

  return <<"EOS";
cn:                        $cn (legacy expiration)
contact_email:             $contact_email$suppress
expires_at:                $expires_at
id:                        $id
EOS
}

sub send_expiration_warning_email
{
  my $self = shift ;

  my $cn_uppercase          = uc($self->cn()) ;
  my $expires_at_date_local = $self->expires_at() ;
  my $contact_email         = $self->contact_email() ;

  return Stanford::Certificate::Notification::send_warning_email(
    $cn_uppercase, $expires_at_date_local, $contact_email
  ) ;

}


sub format_as_request_for_list {
    my $self = shift ;

    my $id            = $self->id() ;
    my $cn            = $self->cn() ;
    my $contact_email = $self->contact_email() ;
    my $expires_at    = $self->expires_at() ;

    my $first_line = sprintf("%6uL  LEGACY CERT       %-40s %-30s \n",
      $id, $cn, $contact_email) ;

    my $second_line = q{ } x 27 . sprintf("(expires %s)\n", $expires_at) ;

    return $first_line . $second_line ;
}

sub format_as_cert_for_list {
    my $self = shift ;

    my $id = $self->id() ;
    my $cn = $self->cn() ;
    my $contact_email = $self->contact_email() ;
    my $expires_at    = $self->expires_at() ;


    return sprintf("%6uL %s %s%s\n", $id, $expires_at, q{}, $cn) ;
}

#certs
#     id expires    cname
#      2 2020-05-30 [S]AddTrust External CA Root
#     9L 2012-03-08 pinnadmin.stanford.edu
#    10L 2012-02-27 pinnadmindev.stanford.edu
#    11L 2015-03-03 pinnadmintrn.stanford.edu
#    12L 2012-03-03 pinnadmintest.stanford.edu


# Returns undef if no certificates found.
sub findById
{
  my ($self) = shift ;

  my ($id) = @_ ;

  my $schema = $self->result_source->schema ;

  my $legs = $schema->resultset('LegacyExpiration')->search(
        { id => $id },
  );

  if ($legs)
  {
    my @all_legs = $legs->all() ;
    if (@all_legs)
    {
      if (scalar(@all_legs) > 1)
      {
        die "found more than one legacy expiration with id '$id'!!)" ;
      }
      else
      {
        my $leg = $all_legs[0] ;
        return $leg ;
      }
    }
  }
  return ;
}

# Returns an array of LegacyExpiration objects; returns the empty array if
# matches found. NOT a method
sub findByCn {
    my ($schema, $cn) = @_ ;

    if (!$cn) {
        croak 'cannot find by CN without CN' ;
    }

    my $legs_rs = $schema->resultset('LegacyExpiration')->search(
          { cn => $cn },
    );

    my @legacies = () ;
    while (my $leg = $legs_rs->next()) {
        push (@legacies, $leg) ;
    }

    return @legacies ;
}

# Note: this does NOT do an update.
sub suppress_notification {
    my $self = shift ;
    $self->suppress_notifications(1) ;
    return ;
}

sub show_email_contact {
  my ($self) = shift ;
  my ($value) = @_ ;

  if (defined($value)) {
      $self->{'show_email_contact'} = $value ;
  }

  return $self->{'show_email_contact'} ;
}

1;
